function [A,U,S,V] = lsd_Asvd(u,v,do_svd)

%[A,U,S,V] = lsd_Asvd(u,v,do_svd)
%
%Generates the matrix A for inversion of the continuity equation to
%determine landslide thicknesses, and also the singular value decomposition
%of A if do_svd = 1 and it doesn't require too much memory.  u and v are
%the horizontal components of the velocity field.
%
%Forward model is -dz/dt = u*dh/dx + v*dh/dy + h*(du/dx + dv/dy)
%
%A has du + dv on main diagonal, -u on diagonal -nrows, v on diagonal -1, 
%-v on diagonal 1, and u on diagonal nrows.  Note:  du and dv are just the
%differences, grid spacing gets included in the solution vector b.
%
%Copyright (C) 2013 Adam M. Booth, [bootha@caltech.edu].
%
%-------------------------------------------------------------------------%
%This program is free software: you can redistribute it and/or modify
%it under the terms of the GNU General Public License as published by
%the Free Software Foundation, either version 3 of the License, or
%any later version.
%
%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU General Public License for more details.
%
%You should have received a copy of the GNU General Public License
%along with this program.  If not, see <http://www.gnu.org/licenses/>.
%-------------------------------------------------------------------------%

%Sizes:
[nrows,ncols] = size(u);
len = nrows*ncols;

%Replace NaN entries with zeros in u and v:
nanmask = ones(nrows,ncols);
nanmask(isnan(u) == 1 | isnan(v) == 1) = NaN;
u = u.*nanmask;
v = v.*nanmask;
u(isnan(u)) = 0;
v(isnan(v)) = 0;

%Reshape arrays to vectors:
u = reshape(u,len,1);
v = reshape(v,len,1);

%Main diagonal using centered finite differences:
vec1 = ones(len,1);
d1cx = spdiags([-1*vec1 vec1],[-nrows nrows],len,len);
d1cx(1:nrows,:) = 0;                        %dudx = 0 on left boundary
d1cx(len-nrows+1:len,:) = 0;                %dudx = 0 on right boundary
d1cy = spdiags([vec1 -1*vec1],[-1 1],len,len);
d1cy(nrows:nrows:len,:) = 0;                %dvdy = 0 on lower boundary
d1cy(1:nrows:len-nrows+1,:) = 0;            %dvdy = 0 on upper boundary
uv0 = d1cx*u + d1cy*v;
%Diagonals -nrows and +nrows from u:
u(1:nrows) = 0;                             %dhdx = 0 on left boundary
uup = zeros(len,1);
uup(nrows+1:len) = u(1:len-nrows);
u(len-nrows+1:len) = 0;                     %dhdx = 0 on right boundary
udn = zeros(len,1);
udn(1:len-nrows) = -1*u(nrows+1:len);
%Diagonals -1 and 1 from v:
v(nrows:nrows:len) = 0;                     %dhdy = 0 on lower boundary
v(1:nrows:len-nrows+1,:) = 0;               %dhdy = 0 on upper boundary
vup = zeros(len,1);
vup(2:len) = -1*v(1:len-1);
vdn = zeros(len,1);
vdn(1:len-1) = v(2:len);

%Put all diagonals together to create A:
A = spdiags([udn vdn uv0 vup uup],[-nrows -1 0 1 nrows],len,len);

if do_svd == 1
    %Singular value decomposition (A = U*S*V')
    [U,S,V] = svd(full(A));
    %Make S sparse to get back some memory, U and V take less memory full:
    S = sparse(S);
else 
    U = NaN; S = NaN; V = NaN;
end

end

