function [h,Lres,Lbeta] = lsd_smooth(A,dzdt,mask,dx,a)

%[h,chi2,b2] = lsd_smooth(A,dzdt,mask,dx,a)
%
%Inverts the continuity equation for landslide thickness, h, using 2nd
%order regularization emphasizing a linearly changing value of h (minimized
%2nd derivative).  Inverse problem is A'*A + a^2*(B'*B) = A'*b.
%
%A = matrix generated by Asvd.m from surface velocity field
%dzdt = array of elevation changes (=(z2 - z1)/dt)
%mask = array of 1s and 0s denoting active landslide pixels
%dx = grid spacing (equal in x- and y-directions)
%a = damping parameter, alpha
%
%h = estimated thickness
%Lres = misfit norm for plotting an Lcurve (||A*h - b||)
%Lbeta = smoothness norm for plotting an Lcurve (||B*h||)
%
%Copyright (C) 2013 Adam M. Booth, [bootha@caltech.edu].
%
%-------------------------------------------------------------------------%
%This program is free software: you can redistribute it and/or modify
%it under the terms of the GNU General Public License as published by
%the Free Software Foundation, either version 3 of the License, or
%any later version.
%
%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU General Public License for more details.
%
%You should have received a copy of the GNU General Public License
%along with this program.  If not, see <http://www.gnu.org/licenses/>.
%-------------------------------------------------------------------------%

%Sizes, shapes, and kill NaNs:
[nrows,ncols] = size(dzdt);
len = nrows*ncols;
dzdt = reshape(dzdt,len,1);
dzdt(isnan(dzdt)) = 0;
mask = reshape(mask,len,1);

%Create smoothing matrix from finite diffrence approximation of laplacian:
[B] = diff2D_2_diag(nrows,ncols);   %does not specify grid spacing or BC's
%Create solution vector:
b = -2*dx*dzdt;
%Invert for h with non-negative constraints:
hmax = 200;                         %set maximum h
hlow = 0.00001;                     %tolerance for h in stable terrain
lb = zeros(len,1);                  %h >= 0 everywhere
ub = hlow + hmax*mask;              %h < hlow in stable terrain only
%Solve using cvx:
cvx_begin
    variable h(len)
    minimize( norm(A*h-b) + a*norm(B*h) );
    subject to 
        lb <= h <= ub   
cvx_end

%Calculate norms for generating an L-curve:
Lres = norm(A*h - b);
Lbeta = norm(B*h);

%Reshape h to matrix:
h = reshape(h,nrows,ncols);

end